package api

import (
	"fmt"
	"net/http"

	"github.com/gin-gonic/gin"

	"git.nemunai.re/nemunaire/reveil/config"
	"git.nemunai.re/nemunaire/reveil/model"
)

func declareGongsRoutes(cfg *config.Config, router *gin.RouterGroup) {
	router.GET("/gongs", func(c *gin.Context) {
		gongs, err := reveil.LoadGongs(cfg)
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		c.JSON(http.StatusOK, gongs)
	})
	router.POST("/gongs", func(c *gin.Context) {
		c.AbortWithStatusJSON(http.StatusNotImplemented, gin.H{"errmsg": "TODO"})
	})

	gongsRoutes := router.Group("/gongs/:tid")
	gongsRoutes.Use(func(c *gin.Context) {
		gongs, err := reveil.LoadGongs(cfg)
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		for _, g := range gongs {
			if g.Id.ToString() == c.Param("tid") {
				c.Set("gong", g)
				c.Next()
				return
			}
		}

		c.AbortWithStatusJSON(http.StatusNotFound, gin.H{"errmsg": "Gong not found"})
	})

	gongsRoutes.GET("", func(c *gin.Context) {
		c.JSON(http.StatusOK, c.MustGet("gong"))
	})
	gongsRoutes.PUT("", func(c *gin.Context) {
		oldgong := c.MustGet("gong").(*reveil.Gong)

		var gong reveil.Gong
		if err := c.ShouldBindJSON(&gong); err != nil {
			c.AbortWithStatusJSON(http.StatusBadRequest, gin.H{"errmsg": err.Error()})
			return
		}

		if gong.Name != oldgong.Name {
			err := oldgong.Rename(gong.Name)
			if err != nil {
				c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": fmt.Sprintf("Unable to rename the gong: %s", err.Error())})
				return
			}
		}

		if gong.Enabled != oldgong.Enabled {
			var err error
			if gong.Enabled {
				err = oldgong.SetDefault(cfg)
			}

			if err != nil {
				c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": fmt.Sprintf("Unable to set the new default gong: %s", err.Error())})
				return
			}
		}

		c.JSON(http.StatusOK, oldgong)
	})
	gongsRoutes.DELETE("", func(c *gin.Context) {
		gong := c.MustGet("gong").(*reveil.Gong)

		err := gong.Remove()
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": fmt.Sprintf("Unable to remove the gong: %s", err.Error())})
			return
		}

		c.JSON(http.StatusOK, nil)
	})
}
